@@ -1,4 +1,3 @@
-from numpy.lib.function_base import append
 import audalign.fingerprint as fingerprint
 from audalign.filehandler import read, find_files, get_shifted_file
 import audalign
@@ -27,6 +26,7 @@ def correcognize(
     sample_rate: int = fingerprint.DEFAULT_FS,
     max_lags: float = None,
     plot: bool = False,
+    technique: str = "correlation",
     **kwargs,
 ):
     """Called from audalign correcognize
@@ -48,13 +48,17 @@ def correcognize(
     assert (
         sample_rate < 200000
     )  # I accidentally used 441000 once... not good, big crash
+    if technique not in ["correlation", "correlation_spectrogram"]:
+        raise ValueError(
+            "technique must be either 'correlation' or 'correlation_spectrogram"
+        )
 
     if max_lags is not None:
-        max_lags = max_lags * sample_rate / 2
+        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique) / 2)
     if filter_matches is None:
         filter_matches = 0.5
     if locality is not None:
-        locality = locality * sample_rate
+        locality = calc_spec_windows(locality, sample_rate, technique)
     if locality_filter_prop is None:
         locality_filter_prop = DEFAULT_LOCALITY_FILTER_PROP
     elif locality_filter_prop > 1.0:
@@ -80,12 +84,27 @@ def correcognize(
     # sos = signal.butter(10, 0.125, "hp", fs=sample_rate, output="sos")
     target_array = signal.sosfilt(sos, target_array)
     against_array = signal.sosfilt(sos, against_array)
+    if technique == "correlation_spectrogram":
+        target_array = fingerprint.fingerprint(
+            target_array, fs=sample_rate, retspec=True
+        ).T
+        target_array = np.clip(target_array, 0, 500)  # never louder than 500
+        against_array = fingerprint.fingerprint(
+            against_array, fs=sample_rate, retspec=True
+        ).T
+        against_array = np.clip(against_array, 0, 500)  # never louder than 500
 
     print("Calculating correlation... ", end="")
     correlation = calc_corrs(
-        against_array, target_array, locality=locality, max_lags=max_lags
+        against_array,
+        target_array,
+        locality=locality,
+        max_lags=max_lags,
+        technique=technique,
     )
 
+    if locality is None:
+        correlation = list(correlation)[0]
     results_list_tuple, scaling_factor = find_maxes(
         correlation=correlation,
         filter_matches=filter_matches,
@@ -93,6 +112,7 @@ def correcognize(
         max_lags=max_lags,
         locality_filter_prop=locality_filter_prop,
         locality=locality,
+        technique=technique,
         **kwargs,
     )
 
@@ -106,9 +126,10 @@ def correcognize(
             arr_b_title=against_file_path,
             scaling_factor=scaling_factor,
             peaks=results_list_tuple,
+            technique=technique,
         )
     elif plot:
-        print("Correlation Plot not compatible with locality")
+        print("\nCorrelation Plot not compatible with locality")
         plot_cor(
             array_a=target_array,
             array_b=against_array,
@@ -126,6 +147,7 @@ def correcognize(
         scaling_factor=scaling_factor,
         sample_rate=sample_rate,
         locality=locality,
+        technique=technique,
     )
 
     t = time.time() - t
@@ -151,19 +173,24 @@ def correcognize_directory(
     plot: bool = False,
     max_lags: float = None,
     _file_audsegs: dict = None,
+    technique: str = "correlation",
     **kwargs,
 ):
     """Called from audalign correcognize_directory"""
     assert (
         sample_rate < 200000
     )  # I accidentally used 441000 once... not good, big crash
+    if technique not in ["correlation", "correlation_spectrogram"]:
+        raise ValueError(
+            "technique must be either 'correlation' or 'correlation_spectrogram"
+        )
 
     t = time.time()
 
     if max_lags is not None:
-        max_lags = max_lags * sample_rate / 2
+        max_lags = int(calc_spec_windows(max_lags, sample_rate, technique) / 2)
     if locality is not None:
-        locality = locality * sample_rate
+        locality = calc_spec_windows(locality, sample_rate, technique)
     if locality_filter_prop is None:
         locality_filter_prop = DEFAULT_LOCALITY_FILTER_PROP
     elif locality_filter_prop > 1.0:
@@ -190,6 +217,11 @@ def correcognize_directory(
         10, fingerprint.threshold, "highpass", fs=sample_rate, output="sos"
     )
     target_array = signal.sosfilt(sos, target_array)
+    if technique == "correlation_spectrogram":
+        target_array = fingerprint.fingerprint(
+            target_array, fs=sample_rate, retspec=True
+        ).T
+        target_array = np.clip(target_array, 0, 500)  # never louder than 500
 
     if type(against_directory) == str:
         against_files = find_files(against_directory)
@@ -212,12 +244,23 @@ def correcognize_directory(
             else:
                 against_array = read(file_path, sample_rate=sample_rate)[0]
             against_array = signal.sosfilt(sos, against_array)
+            if technique == "correlation_spectrogram":
+                against_array = fingerprint.fingerprint(
+                    against_array, fs=sample_rate, retspec=True
+                ).T
+                against_array = np.clip(against_array, 0, 500)  # never louder than 500
 
             print("Calculating correlation... ", end="")
             correlation = calc_corrs(
-                against_array, target_array, locality=locality, max_lags=max_lags
+                against_array,
+                target_array,
+                locality=locality,
+                max_lags=max_lags,
+                technique=technique,
             )
 
+            if locality is None:
+                correlation = list(correlation)[0]
             results_list_tuple, scaling_factor = find_maxes(
                 correlation=correlation,
                 filter_matches=filter_matches,
@@ -225,6 +268,7 @@ def correcognize_directory(
                 max_lags=max_lags,
                 locality_filter_prop=locality_filter_prop,
                 locality=locality,
+                technique=technique,
                 **kwargs,
             )
 
@@ -238,9 +282,10 @@ def correcognize_directory(
                     arr_b_title=file_path,
                     scaling_factor=scaling_factor,
                     peaks=results_list_tuple,
+                    technique=technique,
                 )
             elif plot:
-                print("Correlation Plot not compatible with locality")
+                print("\nCorrelation Plot not compatible with locality")
                 plot_cor(
                     array_a=target_array,
                     array_b=against_array,
@@ -258,6 +303,7 @@ def correcognize_directory(
                 scaling_factor=scaling_factor,
                 sample_rate=sample_rate,
                 locality=locality,
+                technique=technique,
             )
             if single_file_match:
                 file_match = {**file_match, **single_file_match}
@@ -290,6 +336,33 @@ def calc_array_indexes(array, locality):
     return index_list
 
 
+def frames_to_sec(frames, sample_rate):
+    return round(
+        float(frames)
+        / sample_rate
+        * fingerprint.DEFAULT_WINDOW_SIZE
+        * fingerprint.DEFAULT_OVERLAP_RATIO,
+        5,
+    )
+
+
+def calc_spec_windows(seconds, sample_rate, technique):
+    if technique == "correlation":
+        return seconds * sample_rate
+    elif technique == "correlation_spectrogram":
+        return max(
+            int(
+                seconds
+                // (
+                    fingerprint.DEFAULT_WINDOW_SIZE
+                    / sample_rate
+                    * fingerprint.DEFAULT_OVERLAP_RATIO
+                )
+            ),
+            1,
+        )
+
+
 def find_index_arr(against_array, target_array, locality, max_lags):
     index_list_against = calc_array_indexes(against_array, locality)
     index_list_target = calc_array_indexes(target_array, locality)
@@ -306,21 +379,45 @@ def find_index_arr(against_array, target_array, locality, max_lags):
     return index_pairs
 
 
-def calc_corrs(against_array, target_array, locality: float, max_lags: float):
+def calc_corrs(
+    against_array, target_array, locality: float, max_lags: float, technique: str
+):
     if locality is None:
-        yield signal.correlate(against_array, target_array)
+        if technique == "correlation":
+            yield signal.correlate(against_array, target_array)
+        elif technique == "correlation_spectrogram":
+            against_array = against_array.T
+            target_array = target_array.T
+            yield _calc_corrs_spec(against_array, target_array)
     else:
         locality_a = len(against_array) if locality > len(against_array) else locality
         locality_b = len(target_array) if locality > len(target_array) else locality
         indexes = find_index_arr(against_array, target_array, locality, max_lags)
-        for pair in indexes:
-            yield [
-                signal.correlate(
-                    against_array[pair[0] : pair[0] + locality_a],
-                    target_array[pair[1] : pair[1] + locality_b],
-                ),
-                pair,
-            ]
+        if technique == "correlation":
+            for pair in indexes:
+                yield [
+                    signal.correlate(
+                        against_array[pair[0] : pair[0] + locality_a],
+                        target_array[pair[1] : pair[1] + locality_b],
+                    ),
+                    pair,
+                ]
+        elif technique == "correlation_spectrogram":
+            for pair in indexes:
+                yield [
+                    _calc_corrs_spec(
+                        against_array[pair[0] : pair[0] + locality_a].T,
+                        target_array[pair[1] : pair[1] + locality_b].T,
+                    ),
+                    pair,
+                ]
+
+
+def _calc_corrs_spec(against, target):
+    corr_result = signal.correlate(against[0], target[0])
+    for i in range(1, len(against)):
+        corr_result += signal.correlate(against[i], target[i])
+    return corr_result
 
 
 def find_maxes(
@@ -330,16 +427,17 @@ def find_maxes(
     max_lags: float,
     locality_filter_prop: float,
     locality: float,
+    technique: str,
     **kwargs,
 ) -> list:
     print("Finding Local Maximums... ", end="")
     if locality is None:
-        correlation = list(correlation)[0]
         return _find_peaks(
             correlation=correlation,
             filter_matches=filter_matches,
             match_len_filter=match_len_filter,
             max_lags=max_lags,
+            technique=technique,
             **kwargs,
         )
     else:
@@ -353,6 +451,7 @@ def find_maxes(
                     match_len_filter=match_len_filter,
                     max_lags=max_lags,
                     index_pair=i[1],
+                    technique=technique,
                     **kwargs,
                 ),
             ]
@@ -372,16 +471,24 @@ def _find_peaks(
     match_len_filter: int,
     max_lags: float,
     index_pair: tuple = None,
+    technique: str = "correlation",
     **kwargs,
 ):
     """This is where kwargs go. returns zip of peak indices and their heights sorted by height"""
-    scaling_factor = max(correlation) / len(correlation) / SCALING_16_BIT
+    max_corr = np.max(correlation)
+    if technique == "correlation":
+        scaling_factor = max_corr / len(correlation) / SCALING_16_BIT
+    elif technique == "correlation_spectrogram":
+        scaling_factor = (
+            max_corr / len(correlation) / (fingerprint.DEFAULT_WINDOW_SIZE / 2) / 50
+        )  # 200 is about the max of spectrogram
+        # *4 because most frequency bands are quite low, especially with butter filter
     correlation = (
         correlation / np.max(np.abs(correlation), axis=0)
-        if max(correlation) > 0
+        if max_corr > 0
         else correlation
     )
-    # This is quite a bit faster, but I couldn't get it to work. Just wasn't worth it.
+    # This is quite a bit faster, but I couldn't get it to work in a timely manner.
     # Fix this for speedup with locality and max_lags
 
     # if max_lags is not None and index_pair is not None:
@@ -408,11 +515,11 @@ def _find_peaks(
         shift = index_pair[0] - index_pair[1]
         for i, peak in enumerate(peaks_tuples):
             if peak[0] + shift < -max_lags * 2:
-                lag_indexes[0] = i
+                lag_indexes[0] = i + 1
             if peak[0] + shift > max_lags * 2:
                 lag_indexes[1] = i
                 break
-        peaks_tuples = peaks_tuples[lag_indexes[0] : lag_indexes[1]]
+        peaks_tuples = peaks_tuples[lag_indexes[0] + 1 : lag_indexes[1]]
 
     peaks_tuples = sorted(peaks_tuples, key=lambda x: x[1], reverse=True)
 
@@ -483,7 +590,8 @@ def process_results(
     file_name: str,
     scaling_factor: float,
     sample_rate: int,
-    locality=None,
+    locality: float = None,
+    technique: str = "correlation",
 ):
     """Processes peaks and stuff into our regular recognition dictionary"""
 
@@ -493,35 +601,57 @@ def process_results(
     locality_list = []
     locality_seconds = []
     if locality is None:
-        locality_list = [None] * len(results_list)
-        locality_seconds = [None] * len(results_list)
         for result_item in results_list:
             offset_samples.append(result_item[0])
-            offset_seconds.append(result_item[0] / sample_rate)
             peak_heights.append(result_item[1])
+            if technique == "correlation":
+                offset_seconds.append(result_item[0] / sample_rate)
+            elif technique == "correlation_spectrogram":
+                offset_seconds.append(frames_to_sec(result_item[0], sample_rate))
+        locality_list = [None] * len(results_list)
+        locality_seconds = [None] * len(results_list)
     else:
         for result_item in results_list:
             offset_samples.append(result_item[0][0])
-            offset_seconds.append(result_item[0][0] / sample_rate)
+            if technique == "correlation":
+                offset_seconds.append(result_item[0][0] / sample_rate)
+            elif technique == "correlation_spectrogram":
+                offset_seconds.append(frames_to_sec(result_item[0][0], sample_rate))
             peak_heights.append(result_item[0][1])
             locality_list.append(result_item[1])
-            locality_seconds.append(
-                [
-                    (
-                        x[1] / sample_rate * 2,
-                        x[0] / sample_rate * 2,
-                        x[2],
-                    )  # against target or target against???
-                    for x in result_item[1]
-                ]
-            )
+            if technique == "correlation":
+                locality_seconds.append(
+                    [
+                        (
+                            x[1] / sample_rate,
+                            x[0] / sample_rate,
+                            x[2],
+                        )  # target, against, scaling
+                        for x in result_item[1]
+                    ]
+                )
+            elif technique == "correlation_spectrogram":
+                locality_seconds.append(
+                    [
+                        (
+                            frames_to_sec(x[1], sample_rate),
+                            frames_to_sec(x[0], sample_rate),
+                            x[2],
+                        )  # target, against, scaling
+                        for x in result_item[1]
+                    ]
+                )
 
     match = {}
     match[file_name] = {}
 
-    match[file_name]["locality_samples"] = locality_list
+    if technique == "correlation":
+        match[file_name]["locality_samples"] = locality_list
+        match[file_name]["offset_samples"] = offset_samples
+    elif technique == "correlation_spectrogram":
+        match[file_name][audalign.Audalign.LOCALITY] = locality_list
+        match[file_name][audalign.Audalign.OFFSET_SAMPLES] = offset_samples
     match[file_name][audalign.Audalign.LOCALITY_SECS] = locality_seconds
-    match[file_name]["offset_samples"] = offset_samples
     match[file_name][audalign.Audalign.OFFSET_SECS] = offset_seconds
     match[file_name][audalign.Audalign.CONFIDENCE] = peak_heights
     match[file_name]["sample_rate"] = sample_rate
@@ -547,6 +677,7 @@ def plot_cor(
     arr_b_title=None,
     peaks=None,
     scaling_factor=None,
+    technique=None,
 ):
     """
     Really nifty plotter, lots of good information here.
@@ -555,39 +686,54 @@ def plot_cor(
     new_vis_wsize = int(fingerprint.DEFAULT_WINDOW_SIZE / 44100 * sample_rate)
     fig = plt.figure(title)
 
-    fig.add_subplot(3, 2, 1)
-    plt.plot(array_a)
-    plt.xlabel("Sample Index")
-    plt.ylabel("Amplitude")
-    if arr_a_title:
-        plt.title(arr_a_title)
+    if technique == "correlation":
+        fig.add_subplot(3, 2, 1)
+        plt.plot(array_a)
+        plt.xlabel("Sample Index")
+        plt.ylabel("Amplitude")
+        if arr_a_title:
+            plt.title(arr_a_title)
 
-    arr2d_a = fingerprint.fingerprint(
-        array_a, fs=sample_rate, wsize=new_vis_wsize, retspec=True
-    )
-    fig.add_subplot(3, 2, 2)
-    plt.imshow(arr2d_a)  # , cmap=plt.cm.gray)
-    plt.gca().invert_yaxis()
-    if arr_a_title:
-        plt.title(arr_a_title)
+        arr2d_a = fingerprint.fingerprint(
+            array_a, fs=sample_rate, wsize=new_vis_wsize, retspec=True
+        )
+        fig.add_subplot(3, 2, 2)
+        plt.imshow(arr2d_a)  # , cmap=plt.cm.gray)
+        plt.gca().invert_yaxis()
+        if arr_a_title:
+            plt.title(arr_a_title)
 
-    fig.add_subplot(3, 2, 3)
-    plt.plot(array_b)
-    plt.xlabel("Sample Index")
-    plt.ylabel("Amplitude")
-    if arr_b_title:
-        plt.title(arr_b_title)
+        fig.add_subplot(3, 2, 3)
+        plt.plot(array_b)
+        plt.xlabel("Sample Index")
+        plt.ylabel("Amplitude")
+        if arr_b_title:
+            plt.title(arr_b_title)
 
-    arr2d_b = fingerprint.fingerprint(
-        array_b, fs=sample_rate, wsize=new_vis_wsize, retspec=True
-    )
-    fig.add_subplot(3, 2, 4)
-    plt.imshow(arr2d_b)
-    plt.gca().invert_yaxis()
-    if arr_b_title:
-        plt.title(arr_b_title)
+            arr2d_b = fingerprint.fingerprint(
+                array_b, fs=sample_rate, wsize=new_vis_wsize, retspec=True
+            )
+            fig.add_subplot(3, 2, 4)
+            plt.imshow(arr2d_b)
+            plt.gca().invert_yaxis()
+            if arr_b_title:
+                plt.title(arr_b_title)
+    elif technique == "correlation_spectrogram":
+        fig.add_subplot(3, 2, 1)
+        plt.imshow(array_a)  # , cmap=plt.cm.gray)
+        plt.gca().invert_yaxis()
+        if arr_a_title:
+            plt.title(arr_a_title)
+
+        fig.add_subplot(3, 2, 3)
+        plt.imshow(array_b)
+        plt.gca().invert_yaxis()
+        if arr_b_title:
+            plt.title(arr_b_title)
 
     if corr_array is not None:
+        max_cor = np.max(corr_array)
+
         fig.add_subplot(3, 2, 5)
         plt.plot(corr_array)
         if scaling_factor:
@@ -597,8 +743,8 @@ def plot_cor(
         plt.xlabel("Sample Index")
         plt.ylabel("correlation")
         if peaks:
-            indexes = [x[0] + int(len(corr_array) / 2) for x in peaks]
-            heights = [x[1] for x in peaks]
+            indexes = [x[0] / 2 + int(len(corr_array) / 2) for x in peaks]
+            heights = [x[1] * max_cor for x in peaks]
             plt.plot(indexes, heights, "x")
 
     fig.tight_layout()
